
"""
    LogGenerator.py
    
    Desc: this class generates log files in the following format
        ERROR: Feb 2, 2010 at 2000   /mnt/path/file original:123451 current:66655
        NEW ENTRY: Jan 10, 2010 at 1300   /mnt/path/file 12341
        REMOVED: Jan 5, 2011 at 1400 /mnt/path/file 21243
        
        NOTE: the date-time indicates the date-time the job is started

        Log_Feb_2_2010_at_20:20_pg1
        Log_Feb_2_2010_at_20:20_pg2
"""
import time, os

class LogGenerator:
    def __init__(self, logFileSize):
        self.logFileSize = logFileSize*1000
        self.timestamp = ''
        self.entries = [] # example entry: "ERROR: timestamp filename original: <checksum> current: <checksum>"
        self.directory = 'log/' # allow for testing and writing to temp directory
        
    def saveLog(self, entries, timestamp):
        """
            Desc: set error entries, assumed to be a list of strings
                    and timestamp 
        """
        self.timestamp = timestamp
        self.entries = entries
        
        maxBytes = self.logFileSize
        pageNum = 0
        entriesIter = iter(self.entries)
        endEntries = False
        
        while 1:
            filename = "Log_%s_%d" % (self.timestamp, pageNum)
            pageNum += 1
            byteCount = 0
            
            try:
                newLogFile = open(self.directory+'%s' % filename,'w')
            except: # if directory doesn't exist, recreate
                try:
                    os.mkdir(self.directory)
                except:
                    pass
                newLogFile = open(self.directory+'%s' % filename,'w')
            #end
            
            logEntry = filename+'\n'
            
            # iterate through all the entries until there's 
            # either none left or we've reach the max size of the log file
            while 1:
                try:
                    entry = entriesIter.next()
                    byteCount += len(entry)
                    logEntry += entry + "\n"
                except StopIteration:
                    endEntries = True
                   
                if endEntries or byteCount >= maxBytes:
                    if len(logEntry) > len(filename+'\n'):
                        newLogFile.write(logEntry)
                    newLogFile.close()
                    break
            #end
            
            if endEntries: 
                break
            #end
        #end
        self.entries = []
