#!/usr/bin/python
"""
    configParser.py
    
    Desc: this module is used to parse the config file
            and generate a new one if it doens't exist

"""
#from datetime import datetime
import xml.parsers.expat
from xml.dom.minidom import Document

config = {}

class Parser():
    def __init__(self):
        self.parser =  xml.parsers.expat.ParserCreate()
        self.parser.StartElementHandler = self.startElement
        
        self.info = {}
        
        # setup
        self.info['algorithm'] = "sha512"
        self.info['emailHost'] = 'localhost'
        self.info['logFileSize'] = '5' # in KBytes
        
        # schedule
        self.info['runJobEveryXdays'] = '40'
        self.info['jobStartTime'] = "1900"
        
        self.info['workerInfo'] = [] # example entry:{'path':'/mnt/path/','IP':'127.0.0.1', 'port':60000}
        
        # filepath database access info
        self.info['dbURL'] = 'kora'
        self.info['dbName'] = 'testDB'
        self.info['dbUsername'] = 'administrator'
        self.info['dbPassword'] = 'kora5'
        self.info['dbTable'] = 'filepaths'
        self.info['dbFilenameAttrib'] = 'filename'
        self.info['dbFileSizeAttrib'] = 'size'
        
        self.info['notify'] = []
      
        # if no config file exists, create a new one
        try:
            data = open('config.xml')
            self.parse(data)
        except IOError:
            print "Can't find config.xml, generating a new one..."
            self.generateNewConfigInfo()
        #end
        
    def parse(self, data):
        self.parser.Parse(data.read())
    
    def generateNewConfigInfo(self):
        """
            generate new config file in case it's been erased
        """
        newConfig = open('config.xml', 'w')
        
        # Create the minidom document
        doc = Document()
    
        # Create the <config> element
        config = doc.createElement("config")
        doc.appendChild(config)
    
        # Create the <setup> element
        setup = doc.createElement("setup")
        setup.setAttribute("algorithm", "sha512")
        setup.setAttribute("log_size", '5')
        setup.setAttribute("email_host", 'localhost')
        config.appendChild(setup)
        
        # Create a <schedule> element
        schedule = doc.createElement("schedule")
        schedule.setAttribute("run_job_every_x_days", "40")
        schedule.setAttribute("job_start_time", "1900")
        config.appendChild(schedule)
        
        # Create example <worker> elements
        worker = doc.createElement("worker")
        worker.setAttribute("IP", "127.0.0.1")
        worker.setAttribute("port", "60000")
        worker.setAttribute("path", "/mnt/matrix")
        config.appendChild(worker)
        
        worker = doc.createElement("worker")
        worker.setAttribute("IP", "127.0.0.2")
        worker.setAttribute("port", "60000")
        worker.setAttribute("path", "Z:/matrix")
        config.appendChild(worker)
    
        # create the <database> element
        database = doc.createElement("database")
        database.setAttribute("URL", "localhost")
        database.setAttribute("username", "matrix_user")
        database.setAttribute("password", "kora")
        database.setAttribute("dbname", "kora_main")
        database.setAttribute("table", "kora_files")
        database.setAttribute("filename", "filename")
        database.setAttribute("size", "size")
        config.appendChild(database)
        
        # create the <admin> element
        admin = doc.createElement("notify")
        admin.setAttribute("email", "")
        config.appendChild(admin)
        
        newConfig.writelines(doc.toprettyxml("\t"))
        newConfig.close()
        
    def getConfigInfo(self):
        
        return self.info
    
    def printConfigInfo(self):
        print "***CONFIGURATION INFO***\n"
        print "ALGORITHM: %s" % self.info['algorithm']
        print "  MAX LOG SIZE %s KBytes" % (self.info['logFileSize'])
        print "  EMAIL HOST: %s ..." % self.info['emailHost']
        print
        print "  RUN JOB EVERY %s DAYS AT %s" % (self.info['runJobEveryXdays'], self.info['jobStartTime'])
        print 
        print "WORKER INFO..."
        for worker in self.info['workerInfo']:
            print "  IP: %s" % worker['IP']
            print "  PORT: %s" % worker['port']
            print "  PATH: %s" % worker['path']
            print 
        #end
        print
        print "DATABASE INFO..."
        print "  URL: %s" % self.info['dbURL']
        print "  NAME: %s" % self.info['dbUsername']
        print "  USERNAME: %s" % self.info['dbUsername']
        print "  PASSWORD: %s" % self.info['dbPassword']
        print "  TABLE: %s" % self.info['dbTable']
        print "  FILENAME ATTRIB: %s" % self.info['dbFilenameAttrib']
        print "  FILESIZE ATTRIB: %s" % self.info['dbFileSizeAttrib']
        print 
        print "NOTIFY BY EMAIL ON COMPLETION..."
        print 
        for email in self.info['notify']:
            print "  " + email
        #end
        print 
        
    def startElement(self, name, attrs):
        """
            Parse by element name and fill in info 
        """
        name = name.lower()
        
        if name=='setup':
            self.info['algorithm'] = attrs['algorithm']
            self.info['logFileSize'] = attrs['log_size']
            self.info['emailHost'] = attrs['email_host']
        
        elif name=='schedule':
            self.info['runJobEveryXdays'] = attrs['run_job_every_x_days']
            self.info['jobStartTime'] = attrs['job_start_time']
         
            
        elif name=='worker': 
            worker = {}
            worker['IP'] = attrs['IP']
            worker['port'] = attrs['port']
            worker['path'] = attrs['path']
            self.info['workerInfo'].append(worker)
            
        elif name=='database':
            self.info['dbURL'] = attrs['URL']
            self.info['dbUsername'] = attrs['username']
            self.info['dbPassword'] = attrs['password']
            self.info['dbName'] = attrs['dbname']
            self.info['dbTable'] = attrs['table']
            self.info['dbFilenameAttrib'] = attrs['filename']
            self.info['dbFileSize'] = attrs['size']
        
        elif name=='notify':
            self.info['notify'].append(attrs['email'])
        #end
            
# test code
"""
x = Parser()
x.printConfigInfo()
"""